
# q-expansion of newform 8400.2.a.bs, downloaded from the LMFDB on 07 May 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K.gens()[0]**i for i in range(len(poly_data))]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 8400
weight = 2
poly_data = [0, 1]

# The basis for the coefficient ring is just the power basis
# in the root of the defining polynomial above.
hecke_ring_character_values = None
aps_data = [[0], [1], [0], [-1], [-2], [-2], [0], [6], [0], [-6], [-10], [0], [6], [8], [12], [-6], [0], [-6], [-4], [-6], [-14], [-4], [0], [-6], [-2], [-10], [-16], [-4], [10], [10], [4], [-12], [-2], [6], [14], [0], [-22], [16], [24], [0], [-18], [-26], [-10], [-16], [-18], [22], [8], [16], [-8], [-6], [-6], [-30], [-26], [8], [-20], [8], [14], [-30], [0], [-26], [12], [-16], [4], [8], [14], [26], [0], [-16], [-4], [-14], [24], [22], [-16], [4], [20], [16], [-10], [-2], [2], [-6], [4], [10], [-14], [30], [-10], [28], [18], [12], [18], [-36], [-32], [-24], [12], [22], [20], [24], [-30], [-38], [-20], [-34], [-36], [2], [20], [34], [-28], [22], [28], [40], [-42], [-10], [32], [-24], [18], [22], [4], [46], [-44], [0], [38], [-22], [-38], [-12], [48], [-12], [10], [-46], [26], [-20], [16], [14], [4], [-16], [-4], [12], [-10], [-42], [32], [12], [48], [54], [38], [26], [-44], [36], [30], [12], [-10], [-28], [2], [-32], [-56], [-54], [-36], [12], [28], [30], [-44], [30], [18], [-6], [4], [18], [-40], [-32], [-30], [-28], [-52], [22], [-14], [-40], [6], [-18], [-22], [-32], [26], [-26], [32], [-10], [8], [6], [-32], [-56], [4], [24], [-16], [26], [0], [-28], [-50], [60], [-2], [-12], [52], [62], [12], [16], [66], [4], [-12], [64], [-6], [62], [50], [-42], [8], [-56], [-22], [-36], [30], [10], [-60], [10], [0], [8], [-12], [-50], [4], [70], [32], [-2], [58], [-50], [-50], [56], [12], [26], [-12], [46], [-40], [30], [52], [38], [-24], [62], [36], [-20], [42], [30], [6], [28], [12], [-2], [-64], [38], [60], [-36], [-56], [60], [16], [40], [-40], [-42], [-40], [74], [8], [14], [2], [28], [32], [22], [36], [-12], [10], [42], [-8], [-14], [-14], [-70], [68], [54], [62], [-32], [-34], [4], [14], [0], [52], [34], [6], [56], [44], [52], [36], [14], [36], [-10], [-76], [26], [-58], [14], [18], [-56], [-46], [68], [-28], [-14], [-62], [-52], [72], [20], [26], [62], [8], [-44], [-68], [-80], [12], [10], [-62], [60], [12], [74], [66], [68], [-48], [-66], [60], [6], [54], [-38], [26], [36], [-50], [-48], [-2], [30], [-68], [-76], [64], [-30], [-2], [54], [58], [-28], [-88], [-28], [30], [-76], [-18], [-16], [-76], [-54], [38], [-40], [-90], [90], [50], [4], [-84], [48], [-2], [-4], [-78], [8], [-2], [-48], [-20], [84], [50], [56], [-92], [10], [-48], [-6], [-52], [-68], [12], [16], [-74], [-66], [90], [-88], [-50], [-14], [-8], [4], [-50], [-10], [66], [58], [30], [-22], [68], [-18], [70], [48], [-20], [-34], [20], [12], [54], [92], [-30], [100], [42], [-96], [10], [2], [20], [-70], [-50], [-6], [-56], [-36], [-30], [-38], [-8], [-6], [28], [-8], [74], [18], [12], [72], [-24], [-18], [18], [96], [8], [-20], [-10], [54], [-40], [-88], [70], [-32], [36], [-46], [18], [60], [-46], [-54], [8], [-16], [42], [62], [38], [-74], [-76], [-54], [-4], [6], [26], [66], [10], [-34], [-28], [-44], [62], [-110], [32], [76], [-84], [-50], [-44], [-30], [-66], [-72], [50], [-54], [-4], [-48], [-46], [18], [-76], [-80], [92], [12], [-66], [-6], [56], [-76], [-36], [30], [-6], [-6], [-30], [-78], [48], [90], [-14], [-6], [-38], [26], [-80], [-36], [-10], [60], [-10], [8], [26], [96], [86], [84], [-58], [90], [4], [-50], [-102], [28], [-82], [24], [86], [-4], [-76], [-88], [32], [70], [100], [76], [80], [-60], [110], [22], [-56], [112], [74], [30], [98], [56], [-52], [-60], [30], [-48], [-38], [28], [82], [-36], [28], [-82], [-68], [-94]]
